# PiKVM V4 Mini & Plus

<img src="overview.jpg" width="400" />

* [PiKVM V4 Mini Datasheet PDF](v4mini_datasheet.pdf)
* [PiKVM V4 Plus Datasheet PDF](v4plus_datasheet.pdf)


-----
## Installation requirements

* PiKVM V4 Mini or Plus box kit.
* Any reliable HDMI cable.


-----
## Basic setup

1. Unpack the box and connect the PiKVM according to the below:

    | Back side | Front side |
    |-----------|------------|
    | <img src="plus_back.jpg" width="300" /> | <img src="plus_front.jpg" width="300" /> |

    * **HDMI input** (the **bottom right** on the **back** side) and **OTG** port (USB emulation) should be connected to the computer. Optionally, the **[ATX port](atx_board.md)** can be connected to control the power. There should be no USB hub between PiKVM and the computer, as some UEFI/BIOS cannot detect them at the boot stage.

    * Connect **Ethernet** (**bottom right** on the **front** size) to the network using the cable from the kit. The kit includes two Ethernet cables of different colors: one for the network, the second for ATX. Use whichever one you like best.

    * Use the power supply and turn on PiKVM. The Mini comes with USB-C 5V power supply, and Plus with Barrel 12V. Use the appropriate connector on the **front** side. Please note that Plus can also work from 5V, in this case, do not connect a 12V power supply and don't create a significant power load on USB (external flash sticks, cameras, and so on).

    !!! warning
        Do not block the ventilation holes on the sides of the device. This is especially important for the V4 Plus, which has an active cooling.

2. Wait until PiKVM starts and shows a greeting on the built-in display.<br>
    Do not turn off the device until it's fully booted for the first time.

3. **Carefully read and follow [the "First steps" guide](first_steps.md){target=_blank}.**<br>
    It describes how to find PiKVM on the network, how to log in there, change passwords, and so on.<br>
    **Follow the steps described there and come back to this page.**

4. ??? note "Using Fahrenheit instead of Celsius on the OLED"

        ```console
        [root@pikvm ~]# mkdir -p /etc/systemd/system/kvmd-oled.service.d
        ```

        Create file `/etc/systemd/system/kvmd-oled.service.d/override.conf`:

        ```ini
        [Service]
        ExecStart=
        ExecStart=/usr/bin/kvmd-oled --clear-on-exit --fahrenheit
        ```

        Then run `systemctl restart kvmd-oled`. In some cases, if you still do not see Fahrenheit being displayed, reboot the device.

5. **Just reminding again:**

    {!_passwd.md!}

6. Try to manage the computer using PiKVM with the Web Interface.<br>
    Make sure that you get an image and both keyboard and mouse are working.
    If something doesn't work, check out our [FAQ](faq.md) (it's really useful).
    If nothing helped, you can get support in our [Discord chat](https://discord.gg/bpmXfz5).

7. Configure access to PiKVM from the Internet using [port forwarding](https://docs.pikvm.org/port_forwarding)
    or [Tailscale VPN](https://docs.pikvm.org/tailscale), if you need it.

8. **If you are using PiKVM V4 Plus, check out its new exclusive feature: [HDMI Passthrough](pass.md)!**

9. Explore the features of PiKVM using the site's table of contents and have fun!


-----
## ATX connection

[See here](atx_board.md).


-----
## DIP switches

There is a pair of DIP switches on the V4 right side. It is not recommended to change their state while the device is turned on.

| Right side |
|------------|
| <img src="plus_right.jpg" width="450" /> |

* The left switch **[1]**. When pulled down (**ON** state), the advanced backpower protection is used.
  On the default state is **OFF** (up), "diode protection" is used.
  This is required for debugging at the request of technical support. Under normal conditions, it is not necessary to change the mode.

* The right switch **[2]**. When pulled down (**ON** state), the Power Delivery chip is activated on the USB OTG port.
  The default state is **OFF** (up).


-----
## Wi-Fi antenna

The PiKVM V4 has a robust metal case that protects your device from physical damage and electromagnetic interference.
This also means that an external antenna must be used for Wi-Fi.
We recommend the [official Antenna Kit](https://www.raspberrypi.com/products/compute-module-4-antenna-kit)
for Raspberry Pi.

To install the antenna, fix it in the round hole in the back side of the PiKVM,
and connect the wire to the connector on the Compute Module 4,
as described in the [instructions](https://datasheets.raspberrypi.com/cm4/cm4-antenna-kit-product-brief.pdf).

Next, to activate the antenna, add line `dtparam=ant2` to the `/boot/config.txt` file on PiKVM.

[Follow this guide](wifi.md) to configure Wi-Fi in PiKVM OS.


-----
## Internal USB on V4 Plus

The V4 Plus has an internal USB port that can be used at your discretion.
It supports USB 3.0, but by default it only works with USB 2.0 devices due to energy savings.
To enable USB 3.0, it is required to flash the USB controller firmware. It's very easy:

1. Choose `usb3` for USB 3.0 support, or `default` for the default USB 2.0-only firmware:

    ```console
    [root@pikvm ~]# flashrom-vl805 usb3
    ```

2. Perform the soft reboot:

    ```console
    [root@pikvm ~]# reboot
    ```

3. After the soft reboot, perform **reboot by power** (unplug and plug again the power cable).


-----
## Known issues and limitations
* There may be **compatibility** issues with some motherboards (such as **HP** or **DELL**) which are the same as those that exist with PiKVM V2. Not everything is perfect, but if you have already used PiKVM V2 - our new V4 will work great and please you. If there is no image from the BIOS, you can fine-tune the HDMI settings, but it is possible that the Mass Storage devices will not be available in the BIOS.
